## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## JFirstContact Mod by Jeckel. Based on Snaitf's First Contact mod.
##

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup

import CvConfigParser
import CvPath

import ContactUtils

#######SD Tool Kit#######

import SdToolKit
sdEcho         = SdToolKit.sdEcho
sdModInit      = SdToolKit.sdModInit
sdModLoad      = SdToolKit.sdModLoad
sdModSave      = SdToolKit.sdModSave
sdEntityInit   = SdToolKit.sdEntityInit
sdEntityExists = SdToolKit.sdEntityExists
sdGetVal       = SdToolKit.sdGetVal
sdSetVal       = SdToolKit.sdSetVal
sdEntityWipe   = SdToolKit.sdEntityWipe

############################

# globals
gc = CyGlobalContext()


############################################################################


#[JFirstContact Mod]


# This option will turn the mod on or off.
# True - This will activate the JFirstContact Mod.
# False - This will deactivate the JFirstContact Mod.
# Default = True
g_bJFirstContactFeaturesEnabled = True


# ######## These options control how contacts are traded durning First Contact Events. ########


# This option is not implamented yet. Please ignore it.
# What is the percent chance that contact will be transferred from one player
# to another upon first contact.
# This can be any number from 1 to 100.
# The higher the number the great the chance contact will be transferred.
# Default = 35
g_iContactTransferBaseChance = 35


# This option is not implamented yet. Please ignore it.
# What is the max number of contacts one player can transfer to another
# durning a single first contact event.
# 0  will allow all contacts to be transfered durning first contact.
# Default = 0
g_iMaxContactTransfer = 0


# Should the the Diplomacy Popup Screen be shown for each contact traded
# durning a First Contact Event. The Diplomacy Popup Screen will still be
# shown when you physically meet another civ, but extra contacts that are
# gotten will not cause the popup.
# True - The Diplomacy Popup Screen will be shown.
# False - The Diplomacy Popup Screen will NOT be shown.
# Default = False
g_bShowAllFirstContactDiplomacyPopups = False


# ######## The following options control contact trading after First Contact has been established. ########


# Should there be a chance each turn to gain contact with a player you don't know
# as long as they are known by a player you do already know.
# True - 2nd hand contacts can be pass around.
# False -  No passing of contact outside First Contact Events.
# Default = True
g_bCanTrade2ndHandContacts = True


# Should Open Borders agreement be required for the passing of contacts.
# True - No contacts will pass without an Open Borders agreement.
# False - Contacts can pass regardless of Open Borders.
# Default = True
g_bRequireOpenBordersToPassContact = True


# What is the percent chance that a unknown contact will be traded
# through one of your know contacts.
# Value must be a positive integer from 1 to 100.
# All other values will be treated as 1.
# Default = 5
g_iBaseChance2ndHandContactTrade = 5


# What is the max number of 2nd hand contacts that can be randomly traded
# in a single turn.
# Value must be a integer equal to or greater then 0.
# 0 will set no max limit.
# Default = 0
g_iMax2ndHandContactTradesPerTurn = 0


############################################################################



def loadConfigurationData():
        global g_bJFirstContactFeaturesEnabled
        global g_iContactTransferBaseChance
        global g_iMaxContactTransfer
        global g_bShowAllFirstContactDiplomacyPopups
        global g_bCanTrade2ndHandContacts
        global g_bRequireOpenBordersToPassContact
        global g_iBaseChance2ndHandContactTrade
        global g_iMax2ndHandContactTradesPerTurn

        # Load the JFirstContact Mod Config INI file containing all of the configuration information
	config = CvConfigParser.CvConfigParser("JFirstContact Mod Config.ini")
		
	# If the JFirstContact Mod's config.ini file was found then load its values
	if(config != None):
                g_bJFirstContactFeaturesEnabled = config.getboolean("JFirstContact Mod", "JFirstContact Features Enabled", True)
                g_bShowAllFirstContactDiplomacyPopups = config.getboolean("JFirstContact Mod", "Show All First Contact Diplomacy Popups", False)
                g_bCanTrade2ndHandContacts = config.getboolean("JFirstContact Mod", "Can Trade 2nd Hand Contacts", True)
                g_bRequireOpenBordersToPassContact = config.getboolean("JFirstContact Mod", "Require Open Borders To Pass Contact", True)

                g_iContactTransferBaseChance = config.getint("JFirstContact Mod", "Contact Transfer Base Chance", 35)
                if (g_iContactTransferBaseChance < 1) or (g_iContactTransferBaseChance > 100):
                        g_iContactTransferBaseChance = 35

                g_iMaxContactTransfer = config.getint("JFirstContact Mod", "Max Contact Transfer", 0)
                if (g_iMaxContactTransfer < 0):
                        g_iMaxContactTransfer = 0
                
                g_iBaseChance2ndHandContactTrade = config.getint("JFirstContact Mod", "Base Chance 2nd Hand Contact Trade", 5)
                if (g_iBaseChance2ndHandContactTrade < 1) or (g_iBaseChance2ndHandContactTrade > 100):
                        g_iBaseChance2ndHandContactTrade = 5

                g_iMax2ndHandContactTradesPerTurn = config.getint("JFirstContact Mod", "Max 2nd Hand Contact Trades Per Turn", 0)
                if (g_iMax2ndHandContactTradesPerTurn < 0):
                        g_iMax2ndHandContactTradesPerTurn = 0


class JFirstContact:
	def __init__(self):
                # initialize base class

                self.bContactInProgress = False

        ###################################################################
        # These methods produce messages in the log and/or on the screen. #
        ###################################################################

        def alertPlayer(self, iPlayer, message, iColor = 7):

                pPlayer = gc.getPlayer(iPlayer)

                # Return immediately if the player passed in is invalid
		if(pPlayer == None):
			return None
			
		# Return immediately if the player passed in is invalid
		if(pPlayer.isNone()):
			return None

                # If the player isn't human then no need to send a message
                if (not pPlayer.isHuman()):
                        return None

                eventMessageTimeLong = gc.getDefineINT("EVENT_MESSAGE_TIME_LONG")
                szIcon = None
                iFlashX = 0
                iFlashY = 0

                szString = str(message)

                CyInterface().addMessage(iPlayer, True, eventMessageTimeLong,
                                         szString, None, 0, szIcon, ColorTypes(iColor),
                                         iFlashX, iFlashY, True, True)

        def alertIconPlayer(self, iPlayer, message, pObject, szIcon = None, iColor = 7):

                pPlayer = gc.getPlayer(iPlayer)

                # Return immediately if the player passed in is invalid
		if(pPlayer == None):
			return None
			
		# Return immediately if the player passed in is invalid
		if(pPlayer.isNone()):
			return None

                # If the player isn't human then no need to send a message
                if (not pPlayer.isHuman()):
                        return None

                # Return immediately if the object passed in is invalid
		if(pObject == None):
			return None
			
		# Return immediately if the object passed in is invalid
		if(pObject.isNone()):
			return None

                eventMessageTimeLong = gc.getDefineINT("EVENT_MESSAGE_TIME_LONG")

                strMessage = str(message)

                CyInterface().addMessage(iPlayer, True, eventMessageTimeLong,
                                         strMessage, None, 0, szIcon, ColorTypes(iColor),
                                         pObject.getX(), pObject.getY(), True, True)

        def alertPlayerDebug(self, iPlayer, message, value, iColor = 7):

                pPlayer = gc.getPlayer(iPlayer)

                # Return immediately if the player passed in is invalid
		if(pPlayer == None):
			return None
			
		# Return immediately if the player passed in is invalid
		if(pPlayer.isNone()):
			return None

                # If the player isn't human then no need to send a message
                if (not pPlayer.isHuman()):
                        return None

                eventMessageTimeLong = gc.getDefineINT("EVENT_MESSAGE_TIME_LONG")
                szIcon = None
                iFlashX = 0
                iFlashY = 0

                szString = str(message) + " = " + str(value)

                CyInterface().addMessage(iPlayer, True, eventMessageTimeLong,
                                         szString, None, 0, szIcon, ColorTypes(iColor),
                                         iFlashX, iFlashY, True, True)

        ##########################################
        # These are the onEvent Handler methods. #
        ##########################################


        def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList

		pPlayer = gc.getPlayer(iPlayer)

		iTeam = pPlayer.getTeam()
		pTeam = gc.getTeam(iTeam)

		iTransferCount = 0

		iListKnownTeams = ContactUtils.getIntListKnownTeams(iTeam)
		iListUnKnownTeams = ContactUtils.getIntListUnKnownTeams(iTeam)

		loadConfigurationData()

		for iTempTeam in iListKnownTeams:

                        if (iTeam == iTempTeam):
                                continue

                        pTempTeam = gc.getTeam(iTempTeam)

                        if (not pTempTeam.isAlive()):
                                continue

                        iListTempKnownTeams = ContactUtils.getIntListKnownTeams(iTempTeam)

                        for iTempKnownTeam in iListTempKnownTeams:

                                pTempKnownTeam = gc.getTeam(iTempKnownTeam)

                                iRandTransferContact = CyGame().getSorenRandNum(101, "Random Transfer Contact Roll")

                                if (not pTeam.isHasMet(iTempKnownTeam)) and (iRandTransferContact <= g_iBaseChance2ndHandContactTrade):

                                        if (iTransferCount < g_iMax2ndHandContactTradesPerTurn) or (g_iMax2ndHandContactTradesPerTurn == 0):
                                                if (g_bRequireOpenBordersToPassContact):
                                                        if (pTeam.isOpenBorders(iTempTeam)):
                                                                pTeam.meet(iTempKnownTeam, True)
                                                                iTransferCount += 1

                                                else:
                                                        pTeam.meet(iTempKnownTeam, True)
                                                        iTransferCount += 1



        def onFirstContact(self, argsList):
		'Contact'
		iTeamX, iHasMetTeamY = argsList

		pTeamX = gc.getTeam(iTeamX)
                pTeamY = gc.getTeam(iHasMetTeamY)

                iTotalPlayers = gc.getMAX_PLAYERS()

                self.alertPlayer(0, "Running onFirstContact")

                # When random contacts are passed around durning onFirstContact events
                # the onFirstContact event is run each time a contact is passed
                # leading to a loop that only terminates when all or practically all
                # of the contacts are known by everyone that meets one civ.
                # To get around this the variable self.bContactInProgress is init to False.
                # When the for loop below is running it is changed to True so that
                # this if statement will stop the code from starting redundent contact
                # passing loops.
                # Note that for the random exchanging of contacts done in onBeginPlayerTurn
                # the self.bContactInProgress variable is also set to True so as to by pass
                # the below code.
                if (self.bContactInProgress):
                        return None

                loadConfigurationData()

                self.iXTransferCount = 0
                self.iYTransferCount = 0

		for iPlayer in range(iTotalPlayers):

                        self.bContactInProgress = True

                        pPlayer = gc.getPlayer(iPlayer)

                        # Return immediately if the player passed in is invalid
                        if(pPlayer == None):
                                continue
			
                        # Return immediately if the player passed in is invalid
                        if(pPlayer.isNone()):
                                continue

                        # If the player is not alive then return
                        if (not pPlayer.isAlive()):
                                continue

                        iTeamZ = pPlayer.getTeam()
                        pTeamZ = gc.getTeam(iTeamZ)

                        # If the player is TeamX or TeamY then return
                        if (iTeamZ == iTeamX) or (iTeamZ == iHasMetTeamY):
                                continue

                        iRandXTransferContact = CyGame().getSorenRandNum(101, "Random X Transfer Contact Roll")
                        iRandYTransferContact = CyGame().getSorenRandNum(101, "Random Y Transfer Contact Roll")

                        #self.alertPlayerDebug(0, "Civ", pPlayer.getCivilizationDescription(0))
                        #self.alertPlayerDebug(0, "self.iXTransferCount", self.iXTransferCount)
                        #self.alertPlayerDebug(0, "self.iYTransferCount", self.iYTransferCount)
                        #self.alertPlayerDebug(0, "pTeamX.isHasMet(iTeamZ)", pTeamX.isHasMet(iTeamZ))
                        #self.alertPlayerDebug(0, "pTeamY.isHasMet(iTeamZ)", pTeamY.isHasMet(iTeamZ))
                        #self.alertPlayerDebug(0, "RandX", iRandXTransferContact)
                        #self.alertPlayerDebug(0, "RandY", iRandYTransferContact)
                        #self.alertPlayer(0, "--------------------------")
                        #self.alertPlayerDebug(0, "g_iContactTransferBaseChance", g_iContactTransferBaseChance)
                        #self.alertPlayer(0, "============================")

                        if (g_iContactTransferBaseChance >= iRandXTransferContact):
                                if (not pTeamX.isHasMet(iTeamZ)) and (pTeamY.isHasMet(iTeamZ)):
                                        if (g_iMaxContactTransfer == 0) or (g_iMaxContactTransfer > iXTransferCount):

                                                pTeamX.meet(iTeamZ, g_bShowAllFirstContactDiplomacyPopups)
                                                self.iXTransferCount += 1



                        if (g_iContactTransferBaseChance >= iRandYTransferContact):
                                if (pTeamX.isHasMet(iTeamZ)) and (not pTeamY.isHasMet(iTeamZ)):
                                        if (g_iMaxContactTransfer == 0) or (g_iMaxContactTransfer > iYTransferCount):
                                                pTeamY.meet(iTeamZ, g_bShowAllFirstContactDiplomacyPopups)
                                                self.iYTransferCount += 1


                self.iXTransferCount = 0
                self.iYTransferCount = 0
                self.bContactInProgress = False










